{{ vars.pid|imports(base_classes_name="provider-base-classes") }}

class {{ vars.pid|class_header(class_name="provider_class", base_classes_name="provider-base-classes") }}:
    pid_type = "{{ vars.pid.type }}"

{{ vars.pid|extra_code }}