{{ vars.record_metadata|imports }}

class {{ vars.record_metadata|class_header }}:
    """Model for {{ vars.record.class|base_name }} metadata."""

    __tablename__ = "{{ vars.record_metadata.table }}"

    {% if vars.record_metadata.use_versioning %}
    # Enables SQLAlchemy-Continuum versioning
    __versioned__ = {}
    {% endif %}

{{ vars.record_metadata|extra_code }}